@@ -8,17 +8,8 @@ from .enum import Mark, Direction
 class Tick:
     symbol: str
     name: str = ""
-    volume: float = 0
-    open_interest: float = 0
-    last_price: float = 0
-    last_volume: float = 0
-    limit_up: float = 0
-    limit_down: float = 0
-
-    open_price: float = 0
-    high_price: float = 0
-    low_price: float = 0
-    pre_close: float = 0
+    price: float = 0
+    vol: float = 0
 
 @dataclass
 class RawBar:
@@ -61,6 +52,7 @@ class BI:
     symbol: str
     fx_a: FX = None    # 笔开始的分型
     fx_b: FX = None    # 笔结束的分型
+    fxs: List[FX] = None    # 笔内部的分型列表
     direction: Direction = None
     high: float = None
     low: float = None
