@@ -5,7 +5,7 @@
 """
 
 from pyecharts import options as opts
-from pyecharts.charts import HeatMap, Kline, Line, Bar, Scatter, Grid
+from pyecharts.charts import HeatMap, Kline, Line, Bar, Scatter, Grid, Boxplot
 from pyecharts.commons.utils import JsCode
 from typing import List
 import numpy as np
@@ -88,7 +88,7 @@ def kline_pro(kline: List[dict],
     """
     # 配置项设置
     # ------------------------------------------------------------------------------------------------------------------
-    bg_color = "#1f212d"    # 背景
+    bg_color = "#1f212d"  # 背景
     up_color = "#F9293E"
     down_color = "#00aa3b"
 
@@ -116,11 +116,13 @@ def kline_pro(kline: List[dict],
     dz_slider = opts.DataZoomOpts(True, "slider", xaxis_index=[0, 1, 2], pos_top="96%",
                                   pos_bottom="0%", range_start=80, range_end=100)
 
-    yaxis_opts = opts.AxisOpts(is_scale=True, axislabel_opts=opts.LabelOpts(color="#c7c7c7", font_size=8, position="inside"))
+    yaxis_opts = opts.AxisOpts(is_scale=True,
+                               axislabel_opts=opts.LabelOpts(color="#c7c7c7", font_size=8, position="inside"))
 
     grid0_xaxis_opts = opts.AxisOpts(type_="category", grid_index=0, axislabel_opts=label_not_show_opts,
                                      split_number=20, min_="dataMin", max_="dataMax",
-                                     is_scale=True, boundary_gap=False, axisline_opts=opts.AxisLineOpts(is_on_zero=False))
+                                     is_scale=True, boundary_gap=False,
+                                     axisline_opts=opts.AxisLineOpts(is_on_zero=False))
 
     tool_tip_opts = opts.TooltipOpts(
         trigger="axis",
@@ -209,7 +211,7 @@ def kline_pro(kline: List[dict],
         chart_fx.add_xaxis(fx_dts)
         chart_fx.add_yaxis(series_name="FX", y_axis=fx_val, is_selected=False,
                            symbol="circle", symbol_size=6, label_opts=label_not_show_opts,
-                           itemstyle_opts=opts.ItemStyleOpts(color="rgba(152, 147, 193, 1.0)",))
+                           itemstyle_opts=opts.ItemStyleOpts(color="rgba(152, 147, 193, 1.0)", ))
 
         chart_fx.set_global_opts(xaxis_opts=grid0_xaxis_opts, legend_opts=legend_not_show_opts)
         chart_k = chart_k.overlap(chart_fx)
@@ -221,7 +223,7 @@ def kline_pro(kline: List[dict],
         chart_bi.add_xaxis(bi_dts)
         chart_bi.add_yaxis(series_name="BI", y_axis=bi_val, is_selected=True,
                            symbol="diamond", symbol_size=10, label_opts=label_not_show_opts,
-                           itemstyle_opts=opts.ItemStyleOpts(color="rgba(184, 117, 225, 1.0)",),
+                           itemstyle_opts=opts.ItemStyleOpts(color="rgba(184, 117, 225, 1.0)", ),
                            linestyle_opts=opts.LineStyleOpts(width=1.5))
 
         chart_bi.set_global_opts(xaxis_opts=grid0_xaxis_opts, legend_opts=legend_not_show_opts)
@@ -233,7 +235,7 @@ def kline_pro(kline: List[dict],
         chart_xd = Line()
         chart_xd.add_xaxis(xd_dts)
         chart_xd.add_yaxis(series_name="XD", y_axis=xd_val, is_selected=True, symbol="triangle", symbol_size=10,
-                           itemstyle_opts=opts.ItemStyleOpts(color="rgba(37, 141, 54, 1.0)",))
+                           itemstyle_opts=opts.ItemStyleOpts(color="rgba(37, 141, 54, 1.0)", ))
 
         chart_xd.set_global_opts(xaxis_opts=grid0_xaxis_opts, legend_opts=legend_not_show_opts)
         chart_k = chart_k.overlap(chart_xd)
@@ -245,7 +247,7 @@ def kline_pro(kline: List[dict],
             chart_b = Scatter()
             chart_b.add_xaxis(b_dts)
             chart_b.add_yaxis(series_name="BUY", y_axis=b_val, is_selected=False, symbol="arrow", symbol_size=8,
-                              itemstyle_opts=opts.ItemStyleOpts(color="#f31e1e",))
+                              itemstyle_opts=opts.ItemStyleOpts(color="#f31e1e", ))
 
             chart_b.set_global_opts(xaxis_opts=grid0_xaxis_opts, legend_opts=legend_not_show_opts)
             chart_k = chart_k.overlap(chart_b)
@@ -316,3 +318,49 @@ def kline_pro(kline: List[dict],
     grid_chart.add(chart_macd, grid_opts=grid2_opts)
     return grid_chart
 
+
+def box_plot(data: dict,
+             title: str = "箱线图",
+             width: str = "900px",
+             height: str = "680px") -> Boxplot:
+    """
+
+    :param data: 数据
+        样例：
+        data = {
+            "expr 0": [960, 850, 830, 880],
+            "expr 1": [960, 850, 830, 880],
+        }
+    :param title:
+    :param width:
+    :param height:
+    :return:
+    """
+    x_data = []
+    y_data = []
+    for k, v in data.items():
+        x_data.append(k)
+        y_data.append(v)
+
+    init_opts = opts.InitOpts(page_title=title, width=width, height=height)
+
+    chart = Boxplot(init_opts=init_opts)
+    chart.add_xaxis(xaxis_data=x_data)
+    chart.add_yaxis(series_name="", y_axis=y_data)
+    chart.set_global_opts(title_opts=opts.TitleOpts(pos_left="center", title=title),
+                          tooltip_opts=opts.TooltipOpts(trigger="item", axis_pointer_type="shadow"),
+                          xaxis_opts=opts.AxisOpts(
+                              type_="category",
+                              boundary_gap=True,
+                              splitarea_opts=opts.SplitAreaOpts(is_show=False),
+                              axislabel_opts=opts.LabelOpts(formatter="{value}"),
+                              splitline_opts=opts.SplitLineOpts(is_show=False),
+                          ),
+                          yaxis_opts=opts.AxisOpts(
+                              type_="value",
+                              name="",
+                              splitarea_opts=opts.SplitAreaOpts(
+                                  is_show=True, areastyle_opts=opts.AreaStyleOpts(opacity=1)
+                              )
+                          ))
+    return chart
